﻿using System;
using System.Diagnostics;
using System.Timers;
using Ozeki.Common;
using Ozeki.VoIP;

namespace DTMF_IVR_WinForm.Model
{
    class CustomerCall 
    {
        public event EventHandler<GenericEventArgs<string>> NotifyAction;
        public event EventHandler Closing;

        public IPhoneCall PhoneCall { get; private set; }

        private IVRMenuBase currentMenu;

        private MediaManager mediaManager;

        protected Timer repeatTimer;

        public CustomerCall(IPhoneCall call, IVRMenuBase rootmenu)
        {
            mediaManager=new MediaManager(call);

            repeatTimer = new Timer(5000);
            repeatTimer.AutoReset = false;
            repeatTimer.Elapsed += new ElapsedEventHandler(repeatTimer_Elapsed);
            currentMenu = rootmenu;
            SubscribeCurrentMenuEvents();

            PhoneCall = call;
            PhoneCall.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(PhoneCall_CallErrorOccured);
            PhoneCall.DtmfReceived += new EventHandler<VoIPEventArgs<Ozeki.Common.OzTuple<Ozeki.VoIP.Media.VoIPMediaType, DtmfSignal>>>(PhoneCall_DtmfReceived);
            PhoneCall.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(PhoneCall_CallStateChanged);
        }

        private void SubscribeCurrentMenuEvents()
        {
            currentMenu.IntroductionStarting += (currentMenu_IntroductionStarting);
            currentMenu.IntroductionStoped += (currentMenu_IntroductionStoped);
            currentMenu.IntroductionFinished += (currentMenu_IntroductionFinished);
            currentMenu.StepIntoMenu += (currentMenu_StepIntoMenu);
        }

        private void UnSubscribeCurrentMenuEvents()
        {
            currentMenu.IntroductionStarting -= (currentMenu_IntroductionStarting);
            currentMenu.IntroductionStoped -= (currentMenu_IntroductionStoped);
            currentMenu.IntroductionFinished -= (currentMenu_IntroductionFinished);
            currentMenu.StepIntoMenu -= (currentMenu_StepIntoMenu);
        }

        void currentMenu_StepIntoMenu(object sender, GenericEventArgs<IVRMenuBase> e)
        {
            repeatTimer.Stop();
            if (e.Item==null)//If event param is null it's indicates the exit command, close the call.
            {
                PhoneCall.HangUp();
                Close();
                return;
            }
            
            UnSubscribeCurrentMenuEvents();
            currentMenu = e.Item;
            SubscribeCurrentMenuEvents();
            currentMenu.StartIntroduction();
        }

        void currentMenu_IntroductionFinished(object sender, EventArgs e)
        {
            repeatTimer.Start();
        }

        
        void repeatTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            currentMenu.RestartIntroduction();
        }

        void currentMenu_IntroductionStoped(object sender, GenericEventArgs<Ozeki.Media.MediaHandlers.VoIPMediaHandler> e)
        {
            mediaManager.DeAttachVoIPHandlerToCall(e.Item);
        }

        void currentMenu_IntroductionStarting(object sender, GenericEventArgs<Ozeki.Media.MediaHandlers.VoIPMediaHandler> e)
        {
            mediaManager.AttachVoIPHandlerToCall(e.Item);
        }

        void PhoneCall_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            switch (e.Item)
            {
                case CallState.InCall:
                    OnNotifyAction(string.Format("Caller '{0}' is in welcome menu.", PhoneCall.DialInfo));
                    currentMenu.StartIntroduction();

                    break;
                case CallState.Completed:
                    OnNotifyAction(string.Format("{0} hanged up the call.",PhoneCall.DialInfo));
                    currentMenu.StopIntroduction();
                    Close();
                    break;
                case CallState.Error:
                case CallState.Rejected:
                case CallState.Cancelled:
                case CallState.Busy:
                    Close();
                    break;
            }
        }


        private void Close()
        {
            if (Closing != null)
            {
                Closing(this, new EventArgs());
            }
            PhoneCall.CallStateChanged -= PhoneCall_CallStateChanged;
            PhoneCall.CallErrorOccured -= PhoneCall_CallErrorOccured;
            PhoneCall.DtmfReceived -= PhoneCall_DtmfReceived;
            UnSubscribeCurrentMenuEvents();
        }

        void PhoneCall_DtmfReceived(object sender, VoIPEventArgs<Ozeki.Common.OzTuple<Ozeki.VoIP.Media.VoIPMediaType, DtmfSignal>> e)
        {
            OnNotifyAction(string.Format("{0} sent the following DTMF sign: {1}", PhoneCall.DialInfo,e.Item.Item2.Signal));
            currentMenu.CommandReceived(e.Item.Item2.Signal);
        }

        void PhoneCall_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            OnNotifyAction(string.Format("Some error occure in the following {0} call", PhoneCall));
        }


        private void OnNotifyAction(string action)
        {
            if (NotifyAction!= null)
                NotifyAction(this,new GenericEventArgs<string>(action));
        }


        public override string ToString()
        {
            if (PhoneCall==null)
                return base.ToString();
            return PhoneCall.ToString();
        }

       
    }
}
