﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Timers;
using Ozeki.Common;
using Ozeki.Media.MediaHandlers;

namespace DTMF_IVR_WinForm.Model
{
    /// <summary>
    /// The base class of IVR menu system.
    /// </summary>
    public abstract class  IVRMenuBase
    {
        
        public event EventHandler<GenericEventArgs<VoIPMediaHandler>> IntroductionStarting;
        public event EventHandler<GenericEventArgs<VoIPMediaHandler>> IntroductionStoped;
        /// <summary>
        /// Indicates the caller steps to an other menu or exit.
        /// </summary>
        public event EventHandler<GenericEventArgs<IVRMenuBase>> StepIntoMenu;

        /// <summary>
        /// Indicates the introduction reading has finished.
        /// </summary>
        public event EventHandler IntroductionFinished;

        /// <summary>
        /// The current menu's child menus.
        /// </summary>
        public List< IVRMenuBase> ChildMenus;
        public string Introduction { get; protected set; }
        public int Id { get; private set; }

        
        /// <summary>
        /// The current menu's parent menu.
        /// </summary>
        public IVRMenuBase Parent { get; set; }

        public int ParentId { get; private set; }

        /// <summary>
        /// The assigned DTMF key sign.
        /// </summary>
        public int ActionKey { get; private set; }
        

        public IVRMenuBase(IVRMenuElement menu)
        {
            Parent = null;
            ParentId = -1;
            Id = menu.id;
            if (menu.parentId!=null)
                ParentId = (int)menu.parentId;
            if (menu.actionKey != null)
                ActionKey = (int)menu.actionKey;
            else
                ActionKey = -1;
            Introduction = menu.introduction;
            ChildMenus=new List<IVRMenuBase>();
            
        }

        public abstract void StartIntroduction();

        public abstract void StopIntroduction();

        public abstract void RestartIntroduction();

        /// <summary>
        /// Interprets the received DTMF sign.
        /// </summary>
        /// <param name="signal"></param>
        public abstract void CommandReceived(int signal);

        /// <summary>
        /// Indicates the current menu starts the reading.
        /// </summary>
        /// <param name="voiphandler"></param>
        protected void OnIntroductionStarting(VoIPMediaHandler voiphandler)
        {
            if (IntroductionStarting != null)
                IntroductionStarting(this, new GenericEventArgs<VoIPMediaHandler>(voiphandler)); 
        }

        /// <summary>
        /// Indicates the current menu stops the reading.
        /// </summary>
        /// <param name="voiphandler"></param>
        protected void OnIntroductionStoped(VoIPMediaHandler voiphandler)
        {
            if (IntroductionStoped != null)
                IntroductionStoped(this, new GenericEventArgs<VoIPMediaHandler>(voiphandler));
        }

        /// <summary>
        /// Indicates the current menu introduction reading has finished.
        /// </summary>
        protected void OnIntroductionFinished()
        {
            if (IntroductionFinished!=null)
                IntroductionFinished(this,new EventArgs());   
        }

        /// <summary>
        /// Indicates you must step to the desitaniton menu.
        /// </summary>
        /// <param name="destination"></param>
        protected void OnStepIntoMenu(IVRMenuBase destination)
        {
            if (StepIntoMenu != null)
                StepIntoMenu(this, new GenericEventArgs<IVRMenuBase>(destination));   
        }
    }
}
