﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace DTMF_IVR_WinForm.Model
{
    class IVRMenuBuilder
    {
        private IVRSystemDBEntities ivrSystem;

        
        public IVRMenuBuilder()
        {
            try
            {
                ivrSystem = new IVRSystemDBEntities();
            }
            catch (Exception)
            {}
        }

        public IVRMenuBase GetRootMenu()
        {
            List<IVRMenuElement> allelements = GetAllMenuElements();
            if (allelements==null || allelements.Count==0)
            {
                return null;
            }
            IVRMenuBase root = BuildTreeAndGetRoots(allelements);
            return root;
        }

      

        IVRMenuBase BuildTreeAndGetRoots(List<IVRMenuElement> actualObjects)
        {
            Dictionary<int, IVRMenuBase> lookup = new Dictionary<int, IVRMenuBase>();
            actualObjects.ForEach(x => lookup.Add(x.id, createMenuItem(x)));
            foreach (var item in lookup.Values)
            {
                IVRMenuBase proposedParent;
                if (lookup.TryGetValue(item.ParentId, out proposedParent))
                {
                    item.Parent = proposedParent;
                    proposedParent.ChildMenus.Add(item);
                }
            }

            foreach (var item in lookup.Values)
            {
                if (item.ParentId==-1)
                {
                    Debug.WriteLine(item.Introduction);
                    return item;
                }
            }
            return null;
        }


        private IVRMenuBase createMenuItem(IVRMenuElement element)
        {
            IVRMenuBase currentmenu=null;

            switch ((IVRMenuType)element.menutype)
            {
                case IVRMenuType.Playback:
                    currentmenu = new IVRMenuFilePlayback(element);
                    break;
                case IVRMenuType.TextToSpeech:
                    currentmenu = new IVRMenuTextToSpeech(element);
                    break;
                case IVRMenuType.VoicRecording:
                    currentmenu = new IVRMenuVoiceMessageRecorder(element);
                    break;
                default: throw new Exception(string.Format("'{0}'is unknown menu type.", element.menutype));
            }
            return currentmenu;
        }


        private List<IVRMenuElement> all;

        public List<IVRMenuElement> GetAllMenuElements()
        {
            if (all != null)
                return all;

            if (ivrSystem != null)
            {
                var query = from p in ivrSystem.IVRMenuElements
                            select p;
                Debug.WriteLine(query);
                all = query.ToList();
                foreach (var ivrMenuElement in all)
                {
                    Debug.WriteLine(ivrMenuElement.introduction);
                }
                return all;
            }
            else
            {
              all = GetIVRSystemFromCVSFile();
                return all;
            }
        }



        private List<IVRMenuElement> GetIVRSystemFromCVSFile()
        {
            try
            {
                List<IVRMenuElement> elements=new List<IVRMenuElement>();
                using (StreamReader sr = new StreamReader(Path.Combine(Environment.CurrentDirectory,  "DataBase\\IVRSystemSample.csv")))
                {
                    String line;
                    while ((line = sr.ReadLine()) != null)
                    {
                     // MessageBox.Show(line);
                        IVRMenuElement cur = GetMenuElementFromString(line);
                        if (cur !=null)
                        {
                            elements.Add(GetMenuElementFromString(line));    
                        }
                        
                    }
                }
                return elements;
            }
            catch (Exception e)
            {

                MessageBox.Show("The file could not be read: IVRSystemSample.csv.");
                return null;
            }
        }


        private IVRMenuElement GetMenuElementFromString(string line)
        {
            try
            {
                IVRMenuElement current=new IVRMenuElement();
                string[] res= line.Split(';');
                current.id = int.Parse(res[0]);
                current.parentId = int.Parse(res[1]);
                current.introduction = res[2];
                if (res[3]!="NULL")
                {
                    current.actionKey = int.Parse(res[3]);
                }
                current.menutype = int.Parse(res[4]);
                return current;
            }
            catch (Exception)
            {
                return null;
            }            
        }
    }
}
